
# q-expansion of newform 825.2.a.d, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 825
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 1], [-1, 0], [0, 0], [1, 1], [-1, 0], [0, -2], [-1, 1], [5, -1], [-1, 0], [4, 2], [0, 6], [3, 2], [-1, 7], [6, -4], [-1, -6], [-2, 4], [11, 0], [6, -2], [6, 4], [5, -2], [6, -2], [9, -3], [-4, -6], [-2, -4], [-3, 2], [5, -7], [8, -4], [-6, -8], [-6, -8], [10, 0], [-3, 3], [4, -2], [-2, -10], [-2, 12], [-5, -9], [14, 0], [-6, 0], [4, 14], [-2, 14], [-3, -11], [-3, -14], [-5, 12], [9, 2], [8, 8], [-9, -3], [8, -2], [-2, 8], [24, 2], [10, -6], [-11, -6], [-1, 11], [18, -4], [0, -10], [-8, -12], [2, 8], [-6, 12], [4, 14], [-1, -7], [-2, 2], [25, -5], [-9, -7], [-19, -1], [-18, -8], [8, -4], [13, -10], [-28, 2], [-18, 10], [10, 10], [24, 2], [6, 6], [-4, -6], [24, 6], [10, 8], [18, -4], [12, 2], [-20, 0], [-12, 4], [2, -24], [12, -14], [10, 10], [-17, 6], [-27, 0], [4, 2], [2, -8], [15, 9], [-9, -12], [2, 6], [-18, -10], [4, 26], [-18, 12], [12, 16], [-16, 6], [-2, -6], [-10, 10], [38, 2], [6, -20], [-18, -16], [-22, 10], [-21, 15], [0, -8], [-9, 19], [8, 2], [-4, 8], [11, -3], [-26, -12], [-7, 12], [31, -4], [16, 14], [25, -8], [-12, 2], [-14, -12], [42, 0], [24, 10], [16, -16], [4, -16], [-34, 6], [16, -12], [-23, -4], [32, -6], [-16, -22], [-29, 4], [-6, 4], [-32, 6], [-21, 6], [-10, -18], [-23, -13], [-15, -10], [4, 4], [0, -12], [-18, -10], [19, 1], [0, 22], [-8, 28], [22, -8], [-8, 2], [-2, -4], [-2, 2], [15, -5], [-24, -12], [-5, 9], [-11, 15], [-4, 14], [-2, 12], [24, -8], [-14, -24], [-36, -8], [22, -2], [7, 21], [-12, -6], [40, -8], [16, -22], [-24, 0], [-16, -16], [8, 10], [-6, 6], [37, -6], [23, -15], [6, -14], [16, 0], [-13, -7], [17, 14], [5, 17], [-38, 0], [-39, -8], [-4, -40], [-23, -20], [-14, -20], [2, 32]]
